import edu.princeton.cs.algs4.Point2D;
import edu.princeton.cs.algs4.StdDraw;
import edu.princeton.cs.algs4.StdRandom;

public class ShapeFuntimes {
    public static final double CANVAS_SCALE = 10;

    /*
     * Generates a random position within the canvas at least xDim, yDim away from the sides
     */
    public static Point2D randomPos(double xDim, double yDim) {
        return ShapeFuntimes.randomPoint(ShapeFuntimes.CANVAS_SCALE - xDim, ShapeFuntimes.CANVAS_SCALE - yDim);
    }

    /*
     * Gerates a random point using the given ranges for the x- and y-coordinates
     */
    public static Point2D randomPoint(double xRange, double yRange) {
        return new Point2D(StdRandom.uniform(-xRange, xRange), StdRandom.uniform(-yRange, yRange));
    }

    /*
     * Test whether coord ± adjustment is outside the canvas
     */
    public static boolean outsideCanvas(double coord, double adjustment) {
        return coord - adjustment < -ShapeFuntimes.CANVAS_SCALE || coord + adjustment > ShapeFuntimes.CANVAS_SCALE;
    }
    /*
     * Draws lines starting at each vertex of the Polygon p
     * Direction and length based on p's velocity
     */
    public static void drawStreamers(Polygon p) {
        Point2D vel = p.getVelocity();
        StdDraw.setPenColor(StdDraw.ORANGE);
        for (Point2D vertex : p.getVertexes()) {
            StdDraw.line(vertex.x(), vertex.y(), vertex.x() - vel.x(), vertex.y() - vel.y());
        }
    }
    
    public static void main(String[] args) {
        Shape[] shapes = new Shape[20];
        StdDraw.setCanvasSize(800, 800);  // 800-pixel square canvas
        StdDraw.setScale(-CANVAS_SCALE, CANVAS_SCALE);  // sets the dimensions the of the canvas coordinate system
        for (int i = 0; i < shapes.length; i++) {
            // randomly add a Circle or Rectangle to the list of Shapes
            if (StdRandom.uniform() > 0.5) {
                shapes[i] = new Circle(StdRandom.uniform(0.1, 1));
            } else {
                shapes[i] = new Rectangle(StdRandom.uniform(0.1, 1), StdRandom.uniform(0.1, 1));   
            }
        }
        StdDraw.enableDoubleBuffering();  // needed for animation

        while(true) {
            StdDraw.clear();  // clear the previous frame
            for (int i = 0; i < shapes.length; i++) {
                shapes[i].move(0.03);  // move each shape as if 0.03 seconds has passed
                if (shapes[i] instanceof Polygon) {  // check if this shape is a Polygon
                    drawStreamers((Polygon) shapes[i]);
                }
            }
            StdDraw.show();  // now that everything has moved, show the next frame
            StdDraw.pause(30);
        }
    }
}
